@@ -12,7 +12,8 @@ cases_source_dir = os.path.abspath(os.path.join(os.path.dirname(os.getcwd()), "t
 includes_dir = os.path.join(os.getcwd(), "_includes")
 json_examples_dir = os.path.join(os.getcwd(), "_includes", "examples")
 os.makedirs(json_examples_dir, exist_ok=True)
-results_example_dir = os.path.join(os.getcwd(), "assets", "examples")
+asserts_dir = os.path.join(os.getcwd(), "assets")
+results_example_dir = os.path.join(asserts_dir, "examples")
 os.makedirs(results_example_dir, exist_ok=True)
 
 
@@ -20,6 +21,10 @@ readme = "README.md"
 readme_location = os.path.abspath(os.path.join(os.path.dirname(os.getcwd()), readme))
 shutil.copyfile(readme_location, os.path.join(includes_dir, readme))
 
+config_schema = "config_schema.json"
+config_schema_location = os.path.abspath(os.path.join(os.path.dirname(os.getcwd()), config_schema))
+generate_from_filename(config_schema_location, os.path.join(asserts_dir, "config_schema.html"), expand_buttons=True)
+
 
 for case_name in os.listdir(cases_source_dir):
     name, ext = os.path.splitext(case_name)
